@@ -81,7 +81,7 @@ static inline void __memcpy_unaligned(unsigned long d, unsigned long s, long n)
 }
 
 /*
- * Hmm.. Strange. The __asm__ here is there to make gcc use a integer register
+ * Hmm.. Strange. The __asm__ here is there to make gcc use an integer register
  * for the load-store. I don't know why, but it would seem that using a floating
  * point register for the move seems to slow things down (very small difference,
  * though).
@@ -123,3 +123,13 @@ char * bcopy(const char * src, char * dest, size_t n)
 	__memcpy(dest, src, n);
 	return dest;
 }
+
+/*
+ * gcc-2.7.1 and newer generate calls to memset and memcpy.  So we
+ * need to define that here:
+ */
+#ifdef __ELF__
+ asm (".weak memcpy; memcpy = __memcpy");
+#else
+ asm (".weakext memcpy, __memcpy");
+#endif
