@@ -157,16 +157,12 @@ static int set_termios(struct tty_struct * tty, unsigned long arg, int opt)
 		return retval;
 
 	if (opt & TERMIOS_TERMIO) {
-		retval = verify_area(VERIFY_READ, (void *) arg, sizeof(struct termio));
-		if (retval)
-			return retval;
 		memcpy(&tmp_termios, tty->termios, sizeof(struct termios));
-		user_termio_to_kernel_termios(&tmp_termios, (struct termio *) arg);
+		if (user_termio_to_kernel_termios(&tmp_termios, (struct termio *) arg))
+			return -EFAULT;
 	} else {
-		retval = user_termios_to_kernel_termios
-			(&tmp_termios, (struct termios *) arg);
-		if (retval)
-			return retval;
+		if (user_termios_to_kernel_termios(&tmp_termios, (struct termios *) arg))
+			return -EFAULT;
 	}
 
 	if ((opt & TERMIOS_FLUSH) && tty->ldisc.flush_buffer)
@@ -184,12 +180,8 @@ static int set_termios(struct tty_struct * tty, unsigned long arg, int opt)
 
 static int get_termio(struct tty_struct * tty, struct termio * termio)
 {
-	int i;
-
-	i = verify_area(VERIFY_WRITE, termio, sizeof (struct termio));
-	if (i)
-		return i;
-	kernel_termios_to_user_termio(termio, tty->termios);
+	if (kernel_termios_to_user_termio(termio, tty->termios))
+		return -EFAULT;
 	return 0;
 }
 
@@ -382,7 +374,6 @@ int n_tty_ioctl(struct tty_struct * tty, struct file * file,
 {
 	struct tty_struct * real_tty;
 	int retval;
-	int opt = 0;
 
 	if (tty->driver.type == TTY_DRIVER_TYPE_PTY &&
 	    tty->driver.subtype == PTY_TYPE_MASTER)
@@ -411,23 +402,23 @@ int n_tty_ioctl(struct tty_struct * tty, struct file * file,
 			return set_ltchars(real_tty, (struct ltchars *) arg);
 #endif
 		case TCGETS:
-			return kernel_termios_to_user_termios
-				((struct termios *)arg,
-				 real_tty->termios);
+			if (kernel_termios_to_user_termios((struct termios *)arg, real_tty->termios))
+				return -EFAULT;
+			return 0;
 		case TCSETSF:
-			opt |= TERMIOS_FLUSH;
+			return set_termios(real_tty, arg,  TERMIOS_FLUSH);
 		case TCSETSW:
-			opt |= TERMIOS_WAIT;
+			return set_termios(real_tty, arg, TERMIOS_WAIT);
 		case TCSETS:
-			return set_termios(real_tty, arg, opt);
+			return set_termios(real_tty, arg, 0);
 		case TCGETA:
 			return get_termio(real_tty,(struct termio *) arg);
 		case TCSETAF:
-			opt |= TERMIOS_FLUSH;
+			return set_termios(real_tty, arg, TERMIOS_FLUSH | TERMIOS_TERMIO);
 		case TCSETAW:
-			opt |= TERMIOS_WAIT;
+			return set_termios(real_tty, arg, TERMIOS_WAIT | TERMIOS_TERMIO);
 		case TCSETA:
-			return set_termios(real_tty, arg, opt|TERMIOS_TERMIO);
+			return set_termios(real_tty, arg, TERMIOS_TERMIO);
 		case TCXONC:
 			retval = tty_check_change(tty);
 			if (retval)
@@ -488,15 +479,17 @@ int n_tty_ioctl(struct tty_struct * tty, struct file * file,
 				retval = inq_canon(tty);
 			return put_user(retval, (unsigned int *) arg);
 		case TIOCGLCKTRMIOS:
-			return kernel_termios_to_user_termios
-				((struct termios *)arg,
-				 real_tty->termios_locked);
+			if (kernel_termios_to_user_termios((struct termios *)arg, real_tty->termios_locked))
+				return -EFAULT;
+			return 0;
+
 		case TIOCSLCKTRMIOS:
 			if (!suser())
 				return -EPERM;
-			return user_termios_to_kernel_termios
-				(real_tty->termios_locked,
-				 (struct termios *) arg);
+			if (user_termios_to_kernel_termios(real_tty->termios_locked, (struct termios *) arg))
+				return -EFAULT;
+			return 0;
+
 		case TIOCPKT:
 		{
 			int pktmode;
