@@ -1,4 +1,4 @@
-/* $Id: socket.h,v 1.6 1996/04/04 12:51:26 davem Exp $ */
+/* $Id: socket.h,v 1.9 1997/03/17 04:50:50 davem Exp $ */
 #ifndef _ASM_SOCKET_H
 #define _ASM_SOCKET_H
 
@@ -8,14 +8,20 @@
 #define SOL_SOCKET	0xffff
 
 #define SO_DEBUG	0x0001
+#define SO_PASSCRED	0x0002
 #define SO_REUSEADDR	0x0004
 #define SO_KEEPALIVE	0x0008
 #define SO_DONTROUTE	0x0010
 #define SO_BROADCAST	0x0020
+#define SO_PEERCRED	0x0040
 #define SO_LINGER	0x0080
 #define SO_OOBINLINE	0x0100
 /* To add :#define SO_REUSEPORT 0x0200 */
 #define SO_BSDCOMPAT    0x0400
+#define SO_RCVLOWAT     0x0800
+#define SO_SNDLOWAT     0x1000
+#define SO_RCVTIMEO     0x2000
+#define SO_SNDTIMEO     0x4000
 
 /* wha!??? */
 #define SO_DONTLINGER   (~SO_LINGER)  /* Older SunOS compat. hack */
@@ -29,4 +35,9 @@
 #define SO_NO_CHECK	0x000b
 #define SO_PRIORITY	0x000c
 
+/* Security levels - as per NRL IPv6 - don't actually do anything */
+#define SO_SECURITY_AUTHENTICATION		0x000d
+#define SO_SECURITY_ENCRYPTION_TRANSPORT	0x000e
+#define SO_SECURITY_ENCRYPTION_NETWORK		0x000f
+
 #endif /* _ASM_SOCKET_H */
